#include "TObjTArrC.h"

void hserv() {
   // Server program which waits for two clients to connect. It then monitors
   // the sockets and displays the objects it receives. To see how to
   // make a non-blocking server see the script hserv2.C.
   //
   // To run this demo do the following:
   //   - Open three windows
   //   - Start ROOT in all three windows
   //   - Execute in the first window: .x hserv.C
   //   - Execute in the second and third windows: .x hclient.C
   //Author: Fons Rademakers
   
   // Open a server socket looking for connections on a named service or
   // on a specified port.
   //TServerSocket *ss = new TServerSocket("rootserv", kTRUE);
   TServerSocket *ss = new TServerSocket(9090, kTRUE);

   // Accept a connection and return a full-duplex communication socket.
   TSocket *s0 = ss->Accept();

   // tell the clients to start
   s0->Send("go 0");

   // Close the server socket (unless we will use it later to wait for
   // another connection).
   ss->Close();

   // Check some options of socket 0.
   int val;
   s0->GetOption(kSendBuffer, val);
   printf("sendbuffer size: %d\n", val);

   // Get the remote addresses (informational only).
   TInetAddress adr = s0->GetInetAddress();
   adr.Print();


   TMonitor *mon = new TMonitor;
   mon->Add(s0);

   while (1) {
      TMessage *mess;
      TSocket  *s;

      s = mon->Select();

      Int_t ret = s->Recv(mess);
      cout << "recv returned: " << ret << endl;

      Int_t what = mess->What();
      if (what == kMESS_STRING) {
         char str[64];
         mess->ReadString(str, 64);
         printf("Client %d: %s\n", s==s0 ? 0 : 1, str);
         mon->Remove(s);
         if (mon->GetActive() == 0) {
            printf("No more active clients... stopping\n");
            break;
         }
      } else if (what == kMESS_OBJECT) {
			printf("got object of class: %s\n", mess->GetClass()->GetName());
			printf("would have liked: %s\n", TObjTArrC::Class()->GetName());

			TObjTArrC * oa = (TObjTArrC *) mess->ReadObject(mess->GetClass());
//			TObjTArrC * oa = (TObjTArrC *) mess->ReadObject(TObjTArrC::Class());

			if (oa) {
				cout << "got: " << oa->GetSize() << endl;
				Char_t * a;
				a = oa->GetArray();
				for (Int_t i=0; i<oa->GetSize(); ++i) {
					cout << i << " " << (Int_t) a[i] << endl;
				}
		}

      } else {
         printf("*** Unexpected message: %i vs. (STRING == %i, OBJECT == %i) ***\n", what, kMESS_STRING, kMESS_OBJECT);
      }

      delete mess;
   }

   printf("Client 0: bytes recv = %d, bytes sent = %d\n", s0->GetBytesRecv(),
          s0->GetBytesSent());

   // Close the socket.
   s0->Close();
}
